@@ -1,9 +1,13 @@
-use std::io;
-
-use serde::de::{self, Expected, Unexpected};
+//! Deserializes Bencode data.
 
 use crate::error::{Error, Result};
 use crate::read::{self, Read};
+use serde::de::{self, Expected, Unexpected};
+
+#[cfg(all(feature = "alloc", not(feature = "std")))]
+use alloc::{string::String, vec, vec::Vec};
+#[cfg(feature = "std")]
+use std::{io, string::String, vec, vec::Vec};
 
 /// Deserializes an instance of `T` from the bytes of an `io::Read` type.
 ///
@@ -11,6 +15,7 @@ use crate::read::{self, Read};
 ///
 /// Deserialization can fail if the data is not valid, if the data cannot cannot be deserialized
 /// into an instance of `T`, and other IO errors.
+#[cfg(feature = "std")]
 pub fn from_reader<R, T>(r: R) -> Result<T>
 where
     R: io::Read,
@@ -178,6 +183,7 @@ where
     }
 }
 
+#[cfg(feature = "std")]
 impl<R> Deserializer<read::IoRead<R>>
 where
     R: io::Read,
@@ -203,7 +209,7 @@ macro_rules! forward_deserialize_signed_integer {
         {
             self.deserialize_i64(visitor)
         }
-    }
+    };
 }
 
 macro_rules! forward_deserialize_unsigned_integer {
@@ -214,7 +220,7 @@ macro_rules! forward_deserialize_unsigned_integer {
         {
             self.deserialize_u64(visitor)
         }
-    }
+    };
 }
 
 impl<'de, 'a, R: Read<'de>> de::Deserializer<'de> for &'a mut Deserializer<R> {
@@ -581,6 +587,10 @@ where
 mod tests {
     use super::*;
     use serde_derive::Deserialize;
+
+    #[cfg(all(feature = "alloc", not(feature = "std")))]
+    use alloc::collections::BTreeMap;
+    #[cfg(feature = "std")]
     use std::collections::BTreeMap;
 
     #[test]
@@ -653,7 +663,7 @@ mod tests {
 
     #[test]
     fn test_deserialize_struct() -> Result<()> {
-        #[derive(Debug, PartialEq, Eq, Deserialize)]
+        #[derive(Debug, PartialEq, Deserialize)]
         struct S {
             spam: Vec<String>,
         }
